# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Client for interaction with Dataplex."""


import argparse
import io

from googlecloudsdk.api_lib.storage import storage_api
from googlecloudsdk.api_lib.storage import storage_util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import resources
from googlecloudsdk.core import yaml


def GetClientInstance():
  return apis.GetClientInstance('dataplex', 'v1')


def GetMessageModule():
  return apis.GetMessagesModule('dataplex', 'v1')


def WaitForOperation(
    operation, resource, sleep_ms=5000, pre_start_sleep_ms=1000
):
  """Waits for the given google.longrunning.Operation to complete."""
  operation_ref = resources.REGISTRY.Parse(
      operation.name, collection='dataplex.projects.locations.operations'
  )
  poller = waiter.CloudOperationPoller(
      resource, GetClientInstance().projects_locations_operations
  )
  return waiter.WaitFor(
      poller,
      operation_ref,
      'Waiting for [{0}] to finish'.format(operation_ref.RelativeName()),
      sleep_ms=sleep_ms,
      pre_start_sleep_ms=pre_start_sleep_ms,
  )


def CreateLabels(dataplex_resource, args):
  if getattr(args, 'labels', None):
    return dataplex_resource.LabelsValue(
        additionalProperties=[
            dataplex_resource.LabelsValue.AdditionalProperty(
                key=key, value=value
            )
            for key, value in sorted(args.labels.items())
        ]
    )
  return None


def ReadObject(object_url, storage_client=None):
  """Reads an object's content from GCS.

  Args:
    object_url: Can be a local file path or the URL of the object to be read
      from gcs bucket (must have "gs://" prefix).
    storage_client: Storage api client used to read files from gcs.

  Returns:
    A str for the content of the file.

  Raises:
    ObjectReadError:
      If the read of GCS object is not successful.
  """
  if not object_url.startswith('gs://'):
    return yaml.load_path(object_url)
  client = storage_client or storage_api.StorageClient()
  object_ref = storage_util.ObjectReference.FromUrl(object_url)
  try:
    bytes_io = client.ReadObject(object_ref)
    wrapper = io.TextIOWrapper(bytes_io, encoding='utf-8')
    return yaml.load(wrapper.read())
  except Exception as e:
    raise exceptions.BadFileException(
        'Unable to read file {0} due to incorrect file path or insufficient'
        ' read permissions'.format(object_url)
    ) from e


def SnakeToCamel(arg_str):
  """Converts snake case string to camel case."""
  return ''.join(
      word.capitalize() if ind > 0 else word
      for ind, word in enumerate(arg_str.split('_'))
  )


def SnakeToCamelDict(arg_type):
  """Reccursive method to convert all nested snake case dictionary keys to camel case."""
  if isinstance(arg_type, list):
    return [
        SnakeToCamelDict(list_val)
        if isinstance(list_val, (dict, list))
        else list_val
        for list_val in arg_type
    ]
  return {
      SnakeToCamel(key): (
          SnakeToCamelDict(value) if isinstance(value, (dict, list)) else value
      )
      for key, value in arg_type.items()
  }


def FetchExecutionSpecArgs(args_map_as_list):
  """Returns Dataplex task execution spec args as a map of key,value pairs from an input list of strings of format key=value."""
  execution_args_map = dict()
  for arg_entry in args_map_as_list:
    if '=' not in arg_entry:
      raise argparse.ArgumentTypeError(
          "Execution spec argument '{}' should be of the type argKey=argValue."
          .format(arg_entry)
      )
    arg_entry_split = arg_entry.split('=', 1)
    if (
        len(arg_entry_split) < 2
        or len(arg_entry_split[0].strip()) == 0
        or len(arg_entry_split[1]) == 0
    ):
      raise argparse.ArgumentTypeError(
          "Execution spec argument '{}' should be of the format"
          ' argKey=argValue.'.format(arg_entry)
      )
    execution_args_map[arg_entry_split[0]] = arg_entry_split[1]
  return execution_args_map



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Vertex AI endpoints direct raw predict command."""


import base64

from googlecloudsdk.api_lib.ai.endpoints import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import endpoints_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.ai import region_util


def _AddArgs(parser):
  flags.AddEndpointResourceArg(
      parser,
      'to do online direct raw prediction',
      prompt_func=region_util.PromptForOpRegion,
  )
  flags.AddDirectRawPredictInputArg(parser)


def _Run(args, version):
  """Run Vertex AI online direct raw prediction."""
  endpoint_ref = args.CONCEPTS.endpoint.Parse()
  args.region = endpoint_ref.AsDict()['locationsId']
  with endpoint_util.AiplatformEndpointOverrides(
      version, region=args.region, is_prediction=True
  ):
    endpoints_client = client.EndpointsClient(version=version)

    # Decode the base64 encoded input, because it will be encoded by the
    # apitools client.
    input_json = endpoints_util.ReadInputFromArgs(args.json_request)
    input_json['input'] = base64.b64decode(input_json['input']).decode('utf-8')
    if version == constants.GA_VERSION:
      results = endpoints_client.DirectRawPredict(endpoint_ref, input_json)
    else:
      results = endpoints_client.DirectRawPredictBeta(endpoint_ref, input_json)

    # Decode the base64 encoded output.
    results.output = base64.b64decode(results.output)

    if not args.IsSpecified('format'):
      # default format is based on the response.
      args.format = endpoints_util.GetDefaultFormat(
          results.output, key_name='output'
      )
    return results


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
class DirectRawPredictGa(base.Command):
  """Run Vertex AI online direct raw prediction.

     `{command}` sends a direct raw prediction request to Vertex AI endpoint for
     the given input. The request limit is 10MB.

  ## EXAMPLES

  To direct predict against an endpoint ``123'' under project ``example'' in
  region ``us-central1'', run:

    $ {command} 123 --project=example --region=us-central1
    --json-request=input.json
  """

  @staticmethod
  def Args(parser):
    _AddArgs(parser)

  def Run(self, args):
    return _Run(args, constants.GA_VERSION)


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class DirectRawPredictBeta(DirectRawPredictGa):
  """Run Vertex AI online direct raw prediction.

     `{command}` sends a direct raw prediction request to Vertex AI endpoint for
     the given input. The request limit is 10MB.

  ## EXAMPLES

  To direct raw predict against an endpoint ``123'' under project ``example'' in
  region ``us-central1'', run:

    $ {command} 123 --project=example --region=us-central1
    --json-request=input.json
  """

  def Run(self, args):
    return _Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for removing layer7 ddos defense threshold config from security policies."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute.security_policies import flags


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class RemoveLayer7DdosDefenseThresholdConfig(base.UpdateCommand):
  r"""Remove a layer7 ddos defense threshold config from a Compute Engine security policy.

  *{command}* is used to remove layer7 ddos defense threshold configs from security policies.

  ## EXAMPLES

  To remove a layer7 ddos defense threshold config run the following command:

    $ {command} NAME \
       --threshold-config-name=my-threshold-config-name
  """

  SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.SECURITY_POLICY_ARG = flags.SecurityPolicyArgument()
    cls.SECURITY_POLICY_ARG.AddArgument(parser, operation_type='update')
    parser.add_argument(
        '--threshold-config-name',
        required=True,
        help='The name for the threshold config.',
    )

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.SECURITY_POLICY_ARG.ResolveAsResource(args, holder.resources)
    security_policy = client.SecurityPolicy(
        ref=ref, compute_client=holder.client
    )
    existing_security_policy = security_policy.Describe()[0]

    adaptive_protection_config = (
        existing_security_policy.adaptiveProtectionConfig
    )
    if (
        adaptive_protection_config is None
        or adaptive_protection_config.layer7DdosDefenseConfig is None
        or not adaptive_protection_config.layer7DdosDefenseConfig.thresholdConfigs
    ):
      raise exceptions.InvalidArgumentException(
          '--threshold-config-name',
          "There's no existing layer 7 ddos defense threshold config to remove",
      )

    existing_threshold_configs = (
        adaptive_protection_config.layer7DdosDefenseConfig.thresholdConfigs
    )
    new_threshold_configs = [
        threshold_config
        for threshold_config in existing_threshold_configs
        if threshold_config.name != args.threshold_config_name
    ]

    if len(existing_threshold_configs) == len(new_threshold_configs):
      raise exceptions.InvalidArgumentException(
          '--threshold-config-name',
          'layer 7 ddos defense threshold config "%s" does not exist in this'
          ' policy.'
          % args.threshold_config_name,
      )

    adaptive_protection_config.layer7DdosDefenseConfig.thresholdConfigs = (
        new_threshold_configs
    )

    updated_security_policy = holder.client.messages.SecurityPolicy(
        adaptiveProtectionConfig=adaptive_protection_config,
        fingerprint=existing_security_policy.fingerprint,
    )

    return security_policy.Patch(security_policy=updated_security_policy)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute reservation block commands."""



def AddDescribeFlags(parser):
  """Adds flags to the parser for the describe command."""
  parser.add_argument(
      '--block-name',
      metavar='BLOCK_NAME',
      type=str,
      required=True,
      help='The name of the reservation block.')


def AddFullViewFlag(parser):
  help_text = """\
  The view type for the reservation block.
  """
  parser.add_argument(
      '--full-view',
      metavar='FULL_VIEW',
      choices={
          'BLOCK_VIEW_FULL': 'Full detailed view of the reservation block.',
          'BLOCK_VIEW_BASIC': 'Basic default view of the reservation block.',
      },
      default='BLOCK_VIEW_UNSPECIFIED',
      help=help_text,
      required=False,
  )


def AddScopeFlags(parser):
  """Adds scope flag to the parser."""
  parser.add_argument(
      '--scope',
      metavar='SCOPE',
      type=lambda x: x.lower(),
      choices={
          'all': 'Perform maintenance on all hosts in the reservation block.',
          'running': (
              'Perform maintenance only on the hosts in the reservation block'
              ' that have running VMs.'
          ),
          'unused': (
              'Perform maintenance only on the hosts in the reservation block'
              " that don't have running VMs."
          ),
      },
      help='The maintenance scope to set for the reservation block.',
  )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List Compute Engine future reservations."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List Compute Engine future reservations."""

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""\
        table(name,
              specificSkuProperties.totalCount,
              timeWindow.startTime,
              timeWindow.endTime,
              status.procurementStatus,
              zone.basename())
        """)
    parser.display_info.AddUriFunc(utils.MakeGetUriFunc())
    lister.AddZonalListerArgs(parser, hidden=True)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    request_data = lister.ParseZonalFlags(args, holder.resources)

    list_implementation = lister.ZonalLister(
        client, client.apitools_client.futureReservations)

    return lister.Invoke(request_data, list_implementation)


List.detailed_help = {
    'brief':
        'List Compute Engine future reservations.',
    'EXAMPLES':
        """
      To list all Compute Engine future reservations, run:

          $ {command} my-future-reservation
    """
}



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating network peerings."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import batch_helper
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.networks.peerings import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


def _MakeRequests(client, requests, is_async):
  """Helper for making asynchronous or synchronous peering creation requests."""
  if is_async:
    responses, errors = batch_helper.MakeRequests(
        requests=requests,
        http=client.apitools_client.http,
        batch_url=client.batch_url)
    if not errors:
      for operation in responses:
        log.status.write('Creating network peering for [{0}]\n'.format(
            operation.targetLink))
        log.status.write('Monitor its progress at [{0}]\n'.format(
            operation.selfLink))
    else:
      utils.RaiseToolException(errors)
  else:
    # We want to run through the generator that MakeRequests returns in order
    # to actually make the requests.
    responses = client.MakeRequests(requests)

  return responses


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.PREVIEW,
)
class Create(base.Command):
  r"""Create a Compute Engine network peering.

  *{command}* is used to create peerings between virtual networks. Each side of
  a peering association is set up independently. Peering will be active only
  when the configuration from both sides matches.

  ## EXAMPLES

  To create a network peering with the name 'peering-name' between the network
  'local-network' and the network 'peer-network' which exports and imports
  custom routes and subnet routes with public IPs, run:

    $ {command} peering-name \
      --network=local-network \
      --peer-network=peer-network \
      --export-custom-routes \
      --import-custom-routes \
      --export-subnet-routes-with-public-ip \
      --import-subnet-routes-with-public-ip
  """

  _support_stack_type = False

  @classmethod
  def ArgsCommon(cls, parser):

    parser.add_argument('name', help='The name of the peering.')

    parser.add_argument(
        '--network',
        required=True,
        help='The name of the network in the current project to be peered '
        'with the peer network.')

    parser.add_argument(
        '--peer-network',
        required=True,
        help='The name of the network to be peered with the current network.')

    parser.add_argument(
        '--peer-project',
        required=False,
        help='The name of the project for the peer network.  If not specified, '
        'defaults to current project.')

    base.ASYNC_FLAG.AddToParser(parser)

    flags.AddImportCustomRoutesFlag(parser)
    flags.AddExportCustomRoutesFlag(parser)

    flags.AddImportSubnetRoutesWithPublicIpFlag(parser)
    flags.AddExportSubnetRoutesWithPublicIpFlag(parser)

    flags.AddStackType(parser)

    flags.AddUpdateStrategy(parser)

  @classmethod
  def Args(cls, parser):
    cls.ArgsCommon(parser)
    action = actions.DeprecationAction(
        'auto-create-routes',
        warn=(
            'Flag --auto-create-routes is deprecated and will '
            'be removed in a future release.'
        ),
        action='store_true',
    )
    parser.add_argument(
        '--auto-create-routes',
        action=action,
        default=False,
        required=False,
        help=(
            'If set, will automatically create routes for the network peering.'
            ' Flag auto-create-routes is deprecated. Peer network subnet routes'
            ' are always created in a network when peered.'
        ),
    )

  def _CreateNetworkPeeringForRequest(self, client, args):
    peer_network_ref = resources.REGISTRY.Parse(
        args.peer_network,
        params={
            'project': (
                args.peer_project or properties.VALUES.core.project.GetOrFail
            )
        },
        collection='compute.networks',
    )
    network_peering = client.messages.NetworkPeering(
        name=args.name,
        network=peer_network_ref.RelativeName(),
        exchangeSubnetRoutes=True,
    )
    network_peering.exportCustomRoutes = args.export_custom_routes
    network_peering.importCustomRoutes = args.import_custom_routes
    network_peering.exportSubnetRoutesWithPublicIp = (
        args.export_subnet_routes_with_public_ip
    )
    network_peering.importSubnetRoutesWithPublicIp = (
        args.import_subnet_routes_with_public_ip
    )

    if getattr(args, 'stack_type'):
      network_peering.stackType = (
          client.messages.NetworkPeering.StackTypeValueValuesEnum(
              args.stack_type
          )
      )

    if getattr(args, 'update_strategy'):
      network_peering.updateStrategy = (
          client.messages.NetworkPeering.UpdateStrategyValueValuesEnum(
              args.update_strategy
          )
      )

    return network_peering

  def Run(self, args):
    """Issues the request necessary for adding the peering."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    request = client.messages.ComputeNetworksAddPeeringRequest(
        network=args.network,
        networksAddPeeringRequest=client.messages.NetworksAddPeeringRequest(
            networkPeering=self._CreateNetworkPeeringForRequest(client, args)),
        project=properties.VALUES.core.project.GetOrFail())

    requests = [(client.apitools_client.networks, 'AddPeering', request)]
    return _MakeRequests(client, requests, args.async_)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for updating a backend in a backend service."""


from apitools.base.py import encoding

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import exceptions
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.backend_services import backend_flags
from googlecloudsdk.command_lib.compute.backend_services import backend_services_utils
from googlecloudsdk.command_lib.compute.backend_services import flags


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class UpdateBackend(base.UpdateCommand):
  """Update an existing backend of a load balancer or Traffic Director.

  *{command}* updates attributes of a backend that is already associated with a
  backend service. Configurable attributes depend on the load balancing scheme
  and the type of backend (instance group, zonal NEG, serverless NEG, or
  internet NEG). For more information, see [traffic
  distribution](https://cloud.google.com/load-balancing/docs/backend-service#traffic_distribution).
  and the [Failover for Internal TCP/UDP Load Balancing
  overview](https://cloud.google.com/load-balancing/docs/internal/failover-overview).

  To add, remove, or swap backends, use the `gcloud compute backend-services
  remove-backend` and `gcloud compute backend-services add-backend` commands.
  """

  @staticmethod
  def Args(parser):
    flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
    backend_flags.AddDescription(parser)
    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(parser, 'update in')
    backend_flags.AddBalancingMode(parser)
    backend_flags.AddCapacityLimits(parser)
    backend_flags.AddCapacityScalar(parser)
    backend_flags.AddFailover(parser, default=None)
    backend_flags.AddPreference(parser)
    backend_flags.AddCustomMetrics(parser, add_clear_argument=True)

  def _GetGetRequest(self, client, backend_service_ref):
    if backend_service_ref.Collection() == 'compute.regionBackendServices':
      return (client.apitools_client.regionBackendServices,
              'Get',
              client.messages.ComputeRegionBackendServicesGetRequest(
                  backendService=backend_service_ref.Name(),
                  region=backend_service_ref.region,
                  project=backend_service_ref.project))
    return (client.apitools_client.backendServices,
            'Get',
            client.messages.ComputeBackendServicesGetRequest(
                backendService=backend_service_ref.Name(),
                project=backend_service_ref.project))

  def _GetSetRequest(self, client, backend_service_ref, replacement):
    if backend_service_ref.Collection() == 'compute.regionBackendServices':
      return (client.apitools_client.regionBackendServices,
              'Update',
              client.messages.ComputeRegionBackendServicesUpdateRequest(
                  backendService=backend_service_ref.Name(),
                  backendServiceResource=replacement,
                  region=backend_service_ref.region,
                  project=backend_service_ref.project))
    return (client.apitools_client.backendServices,
            'Update',
            client.messages.ComputeBackendServicesUpdateRequest(
                backendService=backend_service_ref.Name(),
                backendServiceResource=replacement,
                project=backend_service_ref.project))

  def _GetGroupRef(self, args, resources, client):
    if args.instance_group:
      return flags.MULTISCOPE_INSTANCE_GROUP_ARG.ResolveAsResource(
          args,
          resources,
          scope_lister=compute_flags.GetDefaultScopeLister(client))
    if args.network_endpoint_group:
      return flags.GetNetworkEndpointGroupArg().ResolveAsResource(
          args,
          resources,
          scope_lister=compute_flags.GetDefaultScopeLister(client))

  def _Modify(self, client, resources, backend_service_ref, args, existing):
    replacement = encoding.CopyProtoMessage(existing)

    backend_to_update = None
    if (
        self.ReleaseTrack() in [base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA]
        and args.service
    ):
      for backend in replacement.backends:
        if backend.service == args.service:
          backend_to_update = backend
          break
      if not backend_to_update:
        raise exceptions.ArgumentError(
            'No backend with name [{0}] is part of the backend '
            'service [{1}].'.format(args.service, backend_service_ref.Name())
        )
    else:
      group_ref = self._GetGroupRef(args, resources, client)
      for backend in replacement.backends:
        # At most one backend will match

        if (
            group_ref.RelativeName()
            == resources.ParseURL(backend.group).RelativeName()
        ):
          backend_to_update = backend
          break

      if not backend_to_update:
        scope_type = None
        scope_name = None
        if hasattr(group_ref, 'zone'):
          scope_type = 'zone'
          scope_name = group_ref.zone
        if hasattr(group_ref, 'region'):
          scope_type = 'region'
          scope_name = group_ref.region
        raise exceptions.ArgumentError(
            'No backend with name [{0}] in {1} [{2}] is part of the backend '
            'service [{3}].'.format(
                group_ref.Name(),
                scope_type,
                scope_name,
                backend_service_ref.Name(),
            )
        )

    if args.description:
      backend_to_update.description = args.description
    elif args.description is not None:
      backend_to_update.description = None

    self._ModifyBalancingModeArgs(
        client, args, backend_to_update, self.ReleaseTrack()
    )

    if backend_to_update is not None and args.failover is not None:
      backend_to_update.failover = args.failover

    if backend_to_update is not None and args.preference is not None:
      backend_to_update.preference = (
          client.messages.Backend.PreferenceValueValuesEnum(args.preference)
      )

    if (
        (
            self.ReleaseTrack() == base.ReleaseTrack.ALPHA
            or self.ReleaseTrack() == base.ReleaseTrack.BETA
        )
        and backend_to_update is not None
        and args.traffic_duration is not None
    ):
      backend_to_update.trafficDuration = (
          client.messages.Backend.TrafficDurationValueValuesEnum(
              args.traffic_duration
          )
      )

    if args.custom_metrics:
      backend_to_update.customMetrics = args.custom_metrics
    if args.custom_metrics_file:
      backend_to_update.customMetrics = args.custom_metrics_file
    if args.clear_custom_metrics:
      backend_to_update.customMetrics = []

    return replacement

  def _ModifyBalancingModeArgs(
      self, client, args, backend_to_update, release_track=None
  ):
    """Update balancing mode fields in backend_to_update according to args.

    Args:
      client: The compute client.
      args: The arguments given to the update-backend command.
      backend_to_update: The backend message to modify.
      release_track: The release track of the command.
    """
    _ModifyBalancingModeArgs(
        client.messages, args, backend_to_update, release_track
    )

  def _ValidateArgs(self, args):
    """Validatest that at least one field to update is specified.

    Args:
      args: The arguments given to the update-backend command.
    """

    if not any([
        args.description is not None,
        args.balancing_mode,
        args.max_utilization is not None,
        args.max_rate is not None,
        args.max_rate_per_instance is not None,
        args.max_rate_per_endpoint is not None,
        args.max_connections is not None,
        args.max_connections_per_instance is not None,
        args.max_connections_per_endpoint is not None,
        args.capacity_scaler is not None,
        args.failover is not None,
        args.preference is not None,
    ]):
      raise exceptions.UpdatePropertyError(
          'At least one property must be modified.')

  def Run(self, args):
    """Issues requests necessary to update backend of the Backend Service."""
    self._ValidateArgs(args)

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    backend_service_ref = (
        flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.ResolveAsResource(
            args,
            holder.resources,
            scope_lister=compute_flags.GetDefaultScopeLister(client)))
    get_request = self._GetGetRequest(client, backend_service_ref)

    backend_service = client.MakeRequests([get_request])[0]

    modified_backend_service = self._Modify(
        client, holder.resources, backend_service_ref, args, backend_service)

    return client.MakeRequests([
        self._GetSetRequest(client, backend_service_ref,
                            modified_backend_service)
    ])


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBackendBeta(UpdateBackend):
  """Update an existing backend in a backend service.

  *{command}* updates a backend that is part of a backend
  service. This is useful for changing the way a backend
  behaves. Example changes that can be made include changing the
  load balancing policy and draining a backend by setting
  its capacity scaler to zero.

  Backends are instance groups or network endpoint groups. One
  of the `--network-endpoint-group` or `--instance-group` flags is required to
  identify the backend that you are modifying. You cannot change
  the instance group or network endpoint group associated with a backend, but
  you can remove a backend and add a new one with `backend-services
  remove-backend` and `backend-services add-backend`.

  The `gcloud compute backend-services edit` command can also
  update a backend if the use of a text editor is desired.

  For more information about the available settings, see
  https://cloud.google.com/load-balancing/docs/backend-service.
  """

  @classmethod
  def Args(cls, parser):
    flags.GLOBAL_REGIONAL_BACKEND_SERVICE_ARG.AddArgument(parser)
    flags.AddInstanceGroupAndNetworkEndpointGroupArgs(
        parser, 'update in', support_inline_service=True
    )
    backend_flags.AddDescription(parser)
    backend_flags.AddBalancingMode(parser, release_track=cls.ReleaseTrack())
    backend_flags.AddCapacityLimits(parser, release_track=cls.ReleaseTrack())
    backend_flags.AddCapacityScalar(parser)
    backend_flags.AddFailover(parser, default=None)
    backend_flags.AddPreference(parser)
    backend_flags.AddCustomMetrics(parser, add_clear_argument=True)
    backend_flags.AddTrafficDuration(parser)

  def _ValidateArgs(self, args):
    """Overrides."""

    if not any([
        args.description is not None,
        args.balancing_mode,
        args.max_utilization is not None,
        args.max_rate is not None,
        args.max_rate_per_instance is not None,
        args.max_rate_per_endpoint is not None,
        args.max_connections is not None,
        args.max_connections_per_instance is not None,
        args.max_connections_per_endpoint is not None,
        args.max_in_flight_requests is not None,
        args.max_in_flight_requests_per_instance is not None,
        args.max_in_flight_requests_per_endpoint is not None,
        args.traffic_duration is not None,
        args.capacity_scaler is not None,
        args.failover is not None,
        args.preference is not None,
        hasattr(args, 'service') and args.service is not None,
    ]):
      raise exceptions.UpdatePropertyError(
          'At least one property must be modified.')


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateBackendAlpha(UpdateBackendBeta):
  """Update an existing backend in a backend service.

  *{command}* updates a backend that is part of a backend
  service. This is useful for changing the way a backend
  behaves. Example changes that can be made include changing the
  load balancing policy and draining a backend by setting
  its capacity scaler to zero.

  Backends are instance groups or network endpoint groups. One
  of the `--network-endpoint-group` or `--instance-group` flags is required to
  identify the backend that you are modifying. You cannot change
  the instance group or network endpoint group associated with a backend, but
  you can remove a backend and add a new one with `backend-services
  remove-backend` and `backend-services add-backend`.

  The `gcloud compute backend-services edit` command can also
  update a backend if the use of a text editor is desired.

  For more information about the available settings, see
  https://cloud.google.com/load-balancing/docs/backend-service.
  """

  def _ValidateArgs(self, args):
    """Overrides."""

    if not any([
        args.description is not None,
        args.balancing_mode,
        args.max_utilization is not None,
        args.max_rate is not None,
        args.max_rate_per_instance is not None,
        args.max_rate_per_endpoint is not None,
        args.max_connections is not None,
        args.max_connections_per_instance is not None,
        args.max_connections_per_endpoint is not None,
        args.max_in_flight_requests is not None,
        args.max_in_flight_requests_per_instance is not None,
        args.max_in_flight_requests_per_endpoint is not None,
        args.traffic_duration is not None,
        args.capacity_scaler is not None,
        args.failover is not None,
        args.preference is not None,
        args.custom_metrics is not None,
        args.custom_metrics_file is not None,
        args.clear_custom_metrics is not None,
    ]):
      raise exceptions.UpdatePropertyError(
          'At least one property must be modified.')


def _ClearMutualExclusiveBackendCapacityThresholds(backend, release_track=None):
  """Initialize the backend's mutually exclusive capacity thresholds."""
  backend.maxRate = None
  backend.maxRatePerInstance = None
  backend.maxConnections = None
  backend.maxConnectionsPerInstance = None
  backend.maxRatePerEndpoint = None
  backend.maxConnectionsPerEndpoint = None
  if (
      release_track == base.ReleaseTrack.ALPHA
      or release_track == base.ReleaseTrack.BETA
  ):
    backend.maxInFlightRequests = None
    backend.maxInFlightRequestsPerInstance = None
    backend.maxInFlightRequestsPerEndpoint = None


def _ModifyBalancingModeArgs(
    messages, args, backend_to_update, release_track=None
):
  """Update balancing mode fields in backend_to_update according to args.

  Args:
    messages: API messages class, determined by the release track.
    args: The arguments given to the update-backend command.
    backend_to_update: The backend message to modify.
    release_track: The release track of the command.
  """

  backend_services_utils.ValidateBalancingModeArgs(
      messages, args, backend_to_update.balancingMode, release_track)

  if args.balancing_mode:
    backend_to_update.balancingMode = (
        messages.Backend.BalancingModeValueValuesEnum(
            args.balancing_mode))

    # If the balancing mode is being changed to RATE (CONNECTION), we must
    # clear the max utilization, max inflight requests and max connections
    # (rate) fields. If the balancing mode is being chagned to IN_FLIGHT,
    # we must clear the max rate and max connections fields otherwise the
    # server will reject the request.
    if (backend_to_update.balancingMode ==
        messages.Backend.BalancingModeValueValuesEnum.RATE):
      backend_to_update.maxUtilization = None
      backend_to_update.maxConnections = None
      backend_to_update.maxConnectionsPerInstance = None
      backend_to_update.maxConnectionsPerEndpoint = None
      if (
          release_track == base.ReleaseTrack.ALPHA
          or release_track == base.ReleaseTrack.BETA
      ):
        backend_to_update.maxInFlightRequests = None
        backend_to_update.maxInFlightRequestsPerInstance = None
        backend_to_update.maxInFlightRequestsPerEndpoint = None
    elif (backend_to_update.balancingMode ==
          messages.Backend.BalancingModeValueValuesEnum.CONNECTION):
      backend_to_update.maxUtilization = None
      backend_to_update.maxRate = None
      backend_to_update.maxRatePerInstance = None
      backend_to_update.maxRatePerEndpoint = None
      if (
          release_track == base.ReleaseTrack.ALPHA
          or release_track == base.ReleaseTrack.BETA
      ):
        backend_to_update.maxInFlightRequests = None
        backend_to_update.maxInFlightRequestsPerInstance = None
        backend_to_update.maxInFlightRequestsPerEndpoint = None
    elif (
        (
            release_track == base.ReleaseTrack.ALPHA
            or release_track == base.ReleaseTrack.BETA
        )
        and backend_to_update.balancingMode
        == messages.Backend.BalancingModeValueValuesEnum.IN_FLIGHT
    ):
      backend_to_update.maxRate = None
      backend_to_update.maxRatePerInstance = None
      backend_to_update.maxRatePerEndpoint = None
      backend_to_update.maxConnections = None
      backend_to_update.maxConnectionsPerInstance = None
      backend_to_update.maxConnectionsPerEndpoint = None

  # Now, we set the parameters that control load balancing.
  # ValidateBalancingModeArgs takes care that the control parameters
  # are compatible with the balancing mode.
  if args.max_utilization is not None:
    backend_to_update.maxUtilization = args.max_utilization

  # max_rate, max_rate_per_instance, max_connections and
  # max_connections_per_instance, max_in_flight_requests
  # and max_in_flight_requests_per_instance are mutually exclusive arguments.
  if args.max_rate is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxRate = args.max_rate
  elif args.max_rate_per_instance is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxRatePerInstance = args.max_rate_per_instance
  elif args.max_connections is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxConnections = args.max_connections
  elif args.max_connections_per_instance is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxConnectionsPerInstance = (
        args.max_connections_per_instance)
  elif args.max_rate_per_endpoint is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxRatePerEndpoint = args.max_rate_per_endpoint
  elif args.max_connections_per_endpoint is not None:
    _ClearMutualExclusiveBackendCapacityThresholds(backend_to_update)
    backend_to_update.maxConnectionsPerEndpoint = (
        args.max_connections_per_endpoint)
  elif (
      release_track == base.ReleaseTrack.ALPHA
      or release_track == base.ReleaseTrack.BETA
  ):
    if args.max_in_flight_requests is not None:
      _ClearMutualExclusiveBackendCapacityThresholds(
          backend_to_update, release_track
      )
      backend_to_update.maxInFlightRequests = args.max_in_flight_requests
    elif args.max_in_flight_requests_per_instance is not None:
      _ClearMutualExclusiveBackendCapacityThresholds(
          backend_to_update, release_track
      )
      backend_to_update.maxInFlightRequestsPerInstance = (
          args.max_in_flight_requests_per_instance
      )
    elif args.max_in_flight_requests_per_endpoint is not None:
      _ClearMutualExclusiveBackendCapacityThresholds(
          backend_to_update, release_track
      )
      backend_to_update.maxInFlightRequestsPerEndpoint = (
          args.max_in_flight_requests_per_endpoint
      )

  if args.capacity_scaler is not None:
    backend_to_update.capacityScaler = args.capacity_scaler



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implements the command to upload Generic artifacts to a repository."""


import os

from apitools.base.py import transfer
from googlecloudsdk.api_lib.artifacts import exceptions as ar_exceptions
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import flags
from googlecloudsdk.command_lib.artifacts import util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources
from googlecloudsdk.core.util import scaled_integer


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Upload(base.Command):
  """Uploads an artifact to a generic repository."""

  api_version = 'v1'

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
    To upload version v0.1.0 of a generic artifact located in /path/to/file/ to a repository in "us-central1":

        $ {command} --location=us-central1 --project=myproject --repository=myrepo \
          --package=mypackage --version=v0.1.0 --source=/path/to/file/

    To upload version v0.1.0 of a generic artifact located in /path/to/file/ to a repository in "us-central1" within a folder structure:

        $ {command} --location=us-central1 --project=myproject --repository=myrepo \
          --package=mypackage --version=v0.1.0 --source=/path/to/file/ --destination-path=folder/file
    """,
  }

  @staticmethod
  def Args(parser):
    """Set up arguments for this command.

    Args:
      parser: An argparse.ArgumentPaser.
    """
    flags.GetRequiredRepoFlag().AddToParser(parser)
    flags.GetSkipExistingFlag().AddToParser(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    group = parser.add_group(mutex=True, required=True)

    parser.add_argument(
        '--package',
        metavar='PACKAGE',
        required=True,
        help='The package to upload.')
    parser.add_argument(
        '--version',
        metavar='VERSION',
        required=True,
        help=(
            'The version of the package. '
            'You cannot overwrite an existing version in the repository.'
        ),
    )
    parser.add_argument(
        '--destination-path',
        metavar='DESTINATION_PATH',
        required=False,
        help=(
            'Use to specify the path to upload a generic '
            'artifact to within a folder structure.'
        ),
    )
    group.add_argument(
        '--source',
        metavar='SOURCE',
        help='The path to the file you are uploading.')
    group.add_argument(
        '--source-directory',
        metavar='SOURCE_DIRECTORY',
        help='The directory you are uploading.')

  def Run(self, args):
    """Run the generic artifact upload command."""

    client = apis.GetClientInstance('artifactregistry', self.api_version)
    messages = client.MESSAGES_MODULE

    source_dir = args.source_directory
    source_file = args.source

    if source_dir and args.async_:
      raise ar_exceptions.InvalidInputValueError(
          'Asynchronous uploads not supported for directories.'
      )

    if source_file and args.skip_existing:
      raise ar_exceptions.InvalidInputValueError(
          'Skip existing is not supported for single file uploads.'
      )
    # Uploading a single file
    if source_file:
      return self.uploadArtifact(args, source_file, client, messages)
    # Uploading a directory
    elif source_dir:
      # If source_dir was specified, expand, normalize and traverse
      # through the directory sending one upload request per file found,
      # preserving the folder structure.
      args.source_directory = os.path.normpath(os.path.expanduser(source_dir))
      if not os.path.isdir(args.source_directory):
        raise ar_exceptions.InvalidInputValueError(
            'Specified path is not an existing directory.'
            )
      log.status.Print('Uploading directory: {}'.format(source_dir))
      for path, _, files in os.walk(args.source_directory):
        for file in files:
          try:
            self.uploadArtifact(
                args, (os.path.join(path, file)), client, messages
            )
          except waiter.OperationError as e:
            if args.skip_existing and 'already exists' in str(e):
              log.warning(
                  'File with the same package and version already exists.'
              )
              continue
            raise

  def uploadArtifact(self, args, file_path, client, messages):
    # Default chunk size to be consistent for uploading to clouds.
    chunksize = scaled_integer.ParseInteger(
        properties.VALUES.storage.upload_chunk_size.Get()
    )
    repo_ref = args.CONCEPTS.repository.Parse()
    # If destination_path was not specified,
    # take the last portion of the file path as the the file name.
    # ie. file path is folder1/folder2/file.txt, the file name is file.txt
    if args.source:
      file_name = os.path.basename(file_path)
      if args.destination_path:
        path = os.path.normpath(args.destination_path)
        file_name = os.path.join(path, os.path.basename(file_path))
    else:
      # ie: "/usr/Desktop/test_generic_folder"
      # remove the prefix from the full file path
      # /usr/Desktop/test_generic_folder/test.txt
      # to get 'test.txt'
      file_name = file_path[len(args.source_directory)+1:]
      if args.destination_path:
        path = os.path.normpath(args.destination_path)
        file_name = os.path.join(path, file_name)

    # Windows uses "\" as its path separator, replace it with "/" to standardize
    # all file resource names.
    file_name = file_name.replace(os.sep, '/')
    request = messages.ArtifactregistryProjectsLocationsRepositoriesGenericArtifactsUploadRequest(
        uploadGenericArtifactRequest=messages.UploadGenericArtifactRequest(
            packageId=args.package,
            versionId=args.version,
            filename=file_name),
        parent=repo_ref.RelativeName())

    mime_type = util.GetMimetype(file_path)
    upload = transfer.Upload.FromFile(
        file_path, mime_type=mime_type, chunksize=chunksize)
    op_obj = client.projects_locations_repositories_genericArtifacts.Upload(
        request, upload=upload)
    op = op_obj.operation
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='artifactregistry.projects.locations.operations')

    # Handle the operation.
    if args.async_:
      return op_ref
    else:
      result = waiter.WaitFor(
          waiter.CloudOperationPollerNoResources(
              client.projects_locations_operations), op_ref,
          'Uploading file: {}'.format(file_name))
      return result



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the parsing input for cloud build v2 API."""

from apitools.base.py import encoding
from googlecloudsdk.api_lib.cloudbuild import cloudbuild_exceptions
from googlecloudsdk.api_lib.cloudbuild.v2 import client_util
from googlecloudsdk.api_lib.cloudbuild.v2 import input_util
from googlecloudsdk.core import yaml


_WORKFLOW_OPTIONS_ENUMS = [
    "options.provenance.enabled",
    "options.provenance.storage",
    "options.provenance.region",
]


def CloudBuildYamlDataToWorkflow(workflow):
  """Convert cloudbuild.yaml file into Workflow message."""
  _WorkflowTransform(workflow)
  _WorkflowValidate(workflow)

  messages = client_util.GetMessagesModule()
  schema_message = encoding.DictToMessage(workflow, messages.Workflow)
  input_util.UnrecognizedFields(schema_message)
  return schema_message


def _WorkflowValidate(workflow):
  """Check that the given workflow has all required fields.

  Args:
    workflow: The user-supplied Cloud Build Workflow YAML.

  Raises:
    InvalidYamlError: If the workflow is invalid.
  """
  if (
      "options" not in workflow
      or "security" not in workflow["options"]
      or "serviceAccount" not in workflow["options"]["security"]
  ):
    raise cloudbuild_exceptions.InvalidYamlError(
        "A service account is required. Specify your user-managed service"
        " account using the options.security.serviceAccount field"
    )


def _WorkflowTransform(workflow):
  """Transform workflow message."""

  if "triggers" in workflow:
    workflow["workflowTriggers"] = workflow.pop("triggers")

  for workflow_trigger in workflow.get("workflowTriggers", []):
    input_util.WorkflowTriggerTransform(workflow_trigger)

  for param_spec in workflow.get("params", []):
    input_util.ParamSpecTransform(param_spec)
    if not param_spec.get("name", ""):
      raise cloudbuild_exceptions.InvalidYamlError(
          "Workflow parameter name is required"
      )
    if (
        param_spec.get("type", "string") != "string"
        or param_spec.get("default", {"type": "STRING"}).get("type") != "STRING"
    ):
      raise cloudbuild_exceptions.InvalidYamlError(
          "Only string are supported for workflow parameters, error at "
          "parameter with name: {}".format(param_spec.get("name"))
      )

  if "pipelineSpec" in workflow:
    workflow["pipelineSpecYaml"] = yaml.dump(
        workflow.pop("pipelineSpec"), round_trip=True
    )
  elif "pipelineRef" in workflow:
    input_util.RefTransform(workflow["pipelineRef"])
  else:
    raise cloudbuild_exceptions.InvalidYamlError(
        "PipelineSpec or PipelineRef is required.")

  for workspace_binding in workflow.get("workspaces", []):
    _WorkspaceBindingTransform(workspace_binding)

  if "options" in workflow and "status" in workflow["options"]:
    popped_status = workflow["options"].pop("status")
    workflow["options"]["statusUpdateOptions"] = popped_status

  for option in _WORKFLOW_OPTIONS_ENUMS:
    input_util.SetDictDottedKeyUpperCase(workflow, option)


def _PipelineSpecTransform(pipeline_spec):
  """Transform pipeline spec message."""

  for pipeline_task in pipeline_spec.get("tasks", []):
    _PipelineTaskTransform(pipeline_task)

  for param_spec in pipeline_spec.get("params", []):
    input_util.ParamSpecTransform(param_spec)

  if "finally" in pipeline_spec:
    finally_tasks = pipeline_spec.pop("finally")
    for task in finally_tasks:
      _PipelineTaskTransform(task)
    pipeline_spec["finallyTasks"] = finally_tasks


def _PipelineTaskTransform(pipeline_task):
  """Transform pipeline task message."""

  if "taskSpec" in pipeline_task:
    popped_task_spec = pipeline_task.pop("taskSpec")
    for param_spec in popped_task_spec.get("params", []):
      input_util.ParamSpecTransform(param_spec)
    pipeline_task["taskSpec"] = {}
    pipeline_task["taskSpec"]["taskSpec"] = popped_task_spec
  elif "taskRef" in pipeline_task:
    input_util.RefTransform(pipeline_task["taskRef"])
    pipeline_task["taskRef"] = pipeline_task.pop("taskRef")

  if "when" in pipeline_task:
    for when_expression in pipeline_task.get("when", []):
      _WhenExpressionTransform(when_expression)
    pipeline_task["whenExpressions"] = pipeline_task.pop("when")

  input_util.ParamDictTransform(pipeline_task.get("params", []))


def _WhenExpressionTransform(when_expression):
  if "operator" in when_expression:
    when_expression["expressionOperator"] = input_util.CamelToSnake(
        when_expression.pop("operator")).upper()


def _WorkspaceBindingTransform(workspace_binding):
  """Transform workspace binding message."""

  if "secretName" in workspace_binding:
    popped_secret = workspace_binding.pop("secretName")
    workspace_binding["secret"] = {}
    workspace_binding["secret"]["secretName"] = popped_secret

  elif "volume" in workspace_binding:
    popped_volume = workspace_binding.pop("volume")
    # Volume Claim Template.
    workspace_binding["volumeClaim"] = {}

    if "storage" in popped_volume:
      storage = popped_volume.pop("storage")
      workspace_binding["volumeClaim"]["storage"] = storage

  else:
    return



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete instant snapshot group command."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.instant_snapshot_groups import flags as isg_flags


DETAILED_HELP = {  # Dict[str, str]
    'brief': 'Delete a Compute Engine instant snapshot group.',
}


def _CommonArgs(parser):
  """A helper function to build args based on different API version."""
  Delete.ISG_ARG = isg_flags.MakeInstantSnapshotGroupArg(plural=True)
  Delete.ISG_ARG.AddArgument(parser, operation_type='delete')


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class Delete(base.DeleteCommand):
  """Delete a Compute Engine instant snapshot group."""

  def _GetCommonScopeNameForRefs(self, refs):
    """Gets common scope for references."""
    has_zone = any(hasattr(ref, 'zone') for ref in refs)
    has_region = any(hasattr(ref, 'region') for ref in refs)

    if has_zone and not has_region:
      return 'zone'
    elif has_region and not has_zone:
      return 'region'
    else:
      return None

  def _CreateDeleteRequests(self, client, isg_refs):
    """Returns a list of delete messages for instant snapshot groups."""

    messages = client.MESSAGES_MODULE
    requests = []
    for isg_ref in isg_refs:
      if isg_ref.Collection() == 'compute.instantSnapshotGroups':
        service = client.instantSnapshotGroups
        request = messages.ComputeInstantSnapshotGroupsDeleteRequest(
            instantSnapshotGroup=isg_ref.Name(),
            project=isg_ref.project,
            zone=isg_ref.zone)
      elif isg_ref.Collection() == 'compute.regionInstantSnapshotGroups':
        service = client.regionInstantSnapshotGroups
        request = messages.ComputeRegionInstantSnapshotGroupsDeleteRequest(
            instantSnapshotGroup=isg_ref.Name(),
            project=isg_ref.project,
            region=isg_ref.region)
      else:
        raise ValueError('Unknown reference type {0}'.format(
            isg_ref.Collection()))

      requests.append((service, 'Delete', request))
    return requests

  @classmethod
  def Args(cls, parser):
    _CommonArgs(parser)

  @classmethod
  def _GetApiHolder(cls, no_http=False):
    return base_classes.ComputeApiHolder(cls.ReleaseTrack())

  def _Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    isg_ref = Delete.ISG_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client),
    )

    scope_name = self._GetCommonScopeNameForRefs(isg_ref)

    utils.PromptForDeletion(isg_ref, scope_name=scope_name, prompt_title=None)

    requests = list(
        self._CreateDeleteRequests(client.apitools_client, isg_ref))

    return client.MakeRequests(requests)

  def Run(self, args):
    return self._Run(args)


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of gcloud Procurement consumer orders list command."""


from googlecloudsdk.api_lib.commerce_procurement import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.commerce_procurement import resource_args

DETAILED_HELP = {
    'EXAMPLES':
        """
        The filter is a query string that can match a selected set of attributes
        with string values. For example:

            $ {command} --filter "display_name=TEST"

        Supported query attributes are the following:

            * `display_name`

        If the query contains special characters other than letters, underscore,
        or digits, the phrase must be quoted with double quotes. For example,
        where the display name needs to be quoted because it contains the
        special character colon:

            $ {command} --filter "display_name=\\"foo:bar\\""

        Queries can be combined with AND, OR, and NOT to form more complex
        queries. They can also be grouped to force a desired evaluation order.
        For example:

            $ {command} --filter "display_name=foo OR display_name=bar"
        """,
}


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class List(base.Command):
  """Returns the List of Order objects resulting from the List API."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: argparse.ArgumentParser to register arguments with.
    """
    resource_args.AddBillingAccountResourceArg(
        parser, 'Parent Cloud Billing account to list orders for.')
    parser.add_argument(
        '--page-size', type=int, help=('Maximum number of resources per page.'))
    parser.add_argument(
        '--page-token',
        help=('Token that specifies the next page in the list.'))
    parser.add_argument(
        '--filter', help=('Filter that limits the list request.'))

  def Run(self, args):
    """Runs the command.

    Args:
      args: The arguments that were provided to this command invocation.

    Returns:
      List of orders and next token if applicable.
    """
    billing_account_ref = args.CONCEPTS.billing_account.Parse()
    result = apis.Orders.List(billing_account_ref.RelativeName(),
                              args.page_size, args.page_token, args.filter)

    # Clear the filter on args so the built-in functionality will not overwrite
    # already filtered result.
    args.filter = ''
    return result


List.detailed_help = DETAILED_HELP



"""Generated message classes for cloudresourcemanager version v2.

Creates, reads, and updates metadata for Google Cloud Platform resource
containers.
"""
# NOTE: This file is autogenerated and should not be edited by hand.


from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'cloudresourcemanager'


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs.  If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted.  Example Policy with
  multiple AuditConfigs:      {       "audit_configs": [         {
  "service": "allServices"           "audit_log_configs": [             {
  "log_type": "DATA_READ",               "exempted_members": [
  "user:jose@example.com"               ]             },             {
  "log_type": "DATA_WRITE",             },             {
  "log_type": "ADMIN_READ",             }           ]         },         {
  "service": "sampleservice.googleapis.com"           "audit_log_configs": [
  {               "log_type": "DATA_READ",             },             {
  "log_type": "DATA_WRITE",               "exempted_members": [
  "user:aliya@example.com"               ]             }           ]         }
  ]     }  For sampleservice, this policy enables DATA_READ, DATA_WRITE and
  ADMIN_READ logging. It also exempts jose@example.com from DATA_READ logging,
  and aliya@example.com from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example:
  {       "audit_log_configs": [         {           "log_type": "DATA_READ",
  "exempted_members": [             "user:jose@example.com"           ]
  },         {           "log_type": "DATA_WRITE",         }       ]     }
  This enables 'DATA_READ' and 'DATA_WRITE' logging, while exempting
  jose@example.com from DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members` with a `role`.

  Fields:
    condition: The condition that is associated with this binding. NOTE: An
      unsatisfied condition will not allow user access via current binding.
      Different bindings, including their conditions, are examined
      independently.
    members: Specifies the identities requesting access for a Cloud Platform
      resource. `members` can have the following values:  * `allUsers`: A
      special identifier that represents anyone who is    on the internet;
      with or without a Google account.  * `allAuthenticatedUsers`: A special
      identifier that represents anyone    who is authenticated with a Google
      account or a service account.  * `user:{emailid}`: An email address that
      represents a specific Google    account. For example,
      `alice@example.com` .   * `serviceAccount:{emailid}`: An email address
      that represents a service    account. For example, `my-other-
      app@appspot.gserviceaccount.com`.  * `group:{emailid}`: An email address
      that represents a Google group.    For example, `admins@example.com`.
      * `domain:{domain}`: The G Suite domain (primary) that represents all
      the    users of that domain. For example, `google.com` or `example.com`.
    role: Role that is assigned to `members`. For example, `roles/viewer`,
      `roles/editor`, or `roles/owner`.
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CloudresourcemanagerFoldersCreateRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersCreateRequest object.

  Fields:
    folder: A Folder resource to be passed as the request body.
    parent: The resource name of the new Folder's parent. Must be of the form
      `folders/{folder_id}` or `organizations/{org_id}`.
  """

  folder = _messages.MessageField('Folder', 1)
  parent = _messages.StringField(2)


class CloudresourcemanagerFoldersDeleteRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersDeleteRequest object.

  Fields:
    foldersId: Part of `name`. the resource name of the Folder to be deleted.
      Must be of the form `folders/{folder_id}`.
  """

  foldersId = _messages.StringField(1, required=True)


class CloudresourcemanagerFoldersGetIamPolicyRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersGetIamPolicyRequest object.

  Fields:
    foldersId: Part of `resource`. REQUIRED: The resource for which the policy
      is being requested. See the operation documentation for the appropriate
      value for this field.
    getIamPolicyRequest: A GetIamPolicyRequest resource to be passed as the
      request body.
  """

  foldersId = _messages.StringField(1, required=True)
  getIamPolicyRequest = _messages.MessageField('GetIamPolicyRequest', 2)


class CloudresourcemanagerFoldersGetRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersGetRequest object.

  Fields:
    foldersId: Part of `name`. The resource name of the Folder to retrieve.
      Must be of the form `folders/{folder_id}`.
  """

  foldersId = _messages.StringField(1, required=True)


class CloudresourcemanagerFoldersListRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersListRequest object.

  Fields:
    pageSize: The maximum number of Folders to return in the response. This
      field is optional.
    pageToken: A pagination token returned from a previous call to
      `ListFolders` that indicates where this listing should continue from.
      This field is optional.
    parent: The resource name of the Organization or Folder whose Folders are
      being listed. Must be of the form `folders/{folder_id}` or
      `organizations/{org_id}`. Access to this method is controlled by
      checking the `resourcemanager.folders.list` permission on the `parent`.
    showDeleted: Controls whether Folders in the DELETE_REQUESTED state should
      be returned. Defaults to false. This field is optional.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3)
  showDeleted = _messages.BooleanField(4)


class CloudresourcemanagerFoldersMoveRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersMoveRequest object.

  Fields:
    foldersId: Part of `name`. The resource name of the Folder to move. Must
      be of the form folders/{folder_id}
    moveFolderRequest: A MoveFolderRequest resource to be passed as the
      request body.
  """

  foldersId = _messages.StringField(1, required=True)
  moveFolderRequest = _messages.MessageField('MoveFolderRequest', 2)


class CloudresourcemanagerFoldersPatchRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersPatchRequest object.

  Fields:
    folder: A Folder resource to be passed as the request body.
    foldersId: Part of `folder.name`. Output only. The resource name of the
      Folder. Its format is `folders/{folder_id}`, for example:
      "folders/1234".
    updateMask: Fields to be updated. Only the `display_name` can be updated.
  """

  folder = _messages.MessageField('Folder', 1)
  foldersId = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class CloudresourcemanagerFoldersSetIamPolicyRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersSetIamPolicyRequest object.

  Fields:
    foldersId: Part of `resource`. REQUIRED: The resource for which the policy
      is being specified. See the operation documentation for the appropriate
      value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  foldersId = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class CloudresourcemanagerFoldersTestIamPermissionsRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersTestIamPermissionsRequest object.

  Fields:
    foldersId: Part of `resource`. REQUIRED: The resource for which the policy
      detail is being requested. See the operation documentation for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  foldersId = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class CloudresourcemanagerFoldersUndeleteRequest(_messages.Message):
  r"""A CloudresourcemanagerFoldersUndeleteRequest object.

  Fields:
    foldersId: Part of `name`. The resource name of the Folder to undelete.
      Must be of the form `folders/{folder_id}`.
    undeleteFolderRequest: A UndeleteFolderRequest resource to be passed as
      the request body.
  """

  foldersId = _messages.StringField(1, required=True)
  undeleteFolderRequest = _messages.MessageField('UndeleteFolderRequest', 2)


class Expr(_messages.Message):
  r"""Represents an expression text. Example:      title: "User account
  presence"     description: "Determines whether the request has a user
  account"     expression: "size(request.user) > 0"

  Fields:
    description: An optional description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.  The application context of the containing message
      determines which well-known feature set of CEL is supported.
    location: An optional string indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: An optional title for the expression, i.e. a short string
      describing its purpose. This can be used e.g. in UIs which allow to
      enter the expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class Folder(_messages.Message):
  r"""A Folder in an Organization's resource hierarchy, used to organize that
  Organization's resources.

  Enums:
    LifecycleStateValueValuesEnum: Output only. The lifecycle state of the
      folder. Updates to the lifecycle_state must be performed via
      DeleteFolder and UndeleteFolder.

  Messages:
    TagsValue: Optional. Input only. Immutable. Tag keys/values directly bound
      to this folder. Each item in the map must be expressed as " : ". For
      example: "123/environment" : "production", "123/costCenter" :
      "marketing"

  Fields:
    createTime: Output only. Timestamp when the Folder was created. Assigned
      by the server.
    displayName: The folder's display name. A folder's display name must be
      unique amongst its siblings, e.g. no two folders with the same parent
      can share the same display name. The display name must start and end
      with a letter or digit, may contain letters, digits, spaces, hyphens and
      underscores and can be no longer than 30 characters. This is captured by
      the regular expression: [\p{L}\p{N}]([\p{L}\p{N}_-
      ]{0,28}[\p{L}\p{N}])?.
    lifecycleState: Output only. The lifecycle state of the folder. Updates to
      the lifecycle_state must be performed via DeleteFolder and
      UndeleteFolder.
    name: Output only. The resource name of the Folder. Its format is
      `folders/{folder_id}`, for example: "folders/1234".
    parent: The Folder's parent's resource name. Updates to the folder's
      parent must be performed via MoveFolder.
    tags: Optional. Input only. Immutable. Tag keys/values directly bound to
      this folder. Each item in the map must be expressed as " : ". For
      example: "123/environment" : "production", "123/costCenter" :
      "marketing"
  """

  class LifecycleStateValueValuesEnum(_messages.Enum):
    r"""Output only. The lifecycle state of the folder. Updates to the
    lifecycle_state must be performed via DeleteFolder and UndeleteFolder.

    Values:
      LIFECYCLE_STATE_UNSPECIFIED: Unspecified state.
      ACTIVE: The normal and active state.
      DELETE_REQUESTED: The folder has been marked for deletion by the user.
    """
    LIFECYCLE_STATE_UNSPECIFIED = 0
    ACTIVE = 1
    DELETE_REQUESTED = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class TagsValue(_messages.Message):
    r"""Optional. Input only. Immutable. Tag keys/values directly bound to
    this folder. Each item in the map must be expressed as " : ". For example:
    "123/environment" : "production", "123/costCenter" : "marketing"

    Messages:
      AdditionalProperty: An additional property for a TagsValue object.

    Fields:
      additionalProperties: Additional properties of type TagsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a TagsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  displayName = _messages.StringField(2)
  lifecycleState = _messages.EnumField('LifecycleStateValueValuesEnum', 3)
  name = _messages.StringField(4)
  parent = _messages.StringField(5)
  tags = _messages.MessageField('TagsValue', 6)


class FolderOperation(_messages.Message):
  r"""Metadata describing a long running folder operation

  Enums:
    OperationTypeValueValuesEnum: The type of this operation.

  Fields:
    destinationParent: The resource name of the folder or organization we are
      either creating the folder under or moving the folder to.
    displayName: The display name of the folder.
    operationType: The type of this operation.
    sourceParent: The resource name of the folder's parent. Only applicable
      when the operation_type is MOVE.
  """

  class OperationTypeValueValuesEnum(_messages.Enum):
    r"""The type of this operation.

    Values:
      OPERATION_TYPE_UNSPECIFIED: Operation type not specified.
      CREATE: A create folder operation.
      MOVE: A move folder operation.
    """
    OPERATION_TYPE_UNSPECIFIED = 0
    CREATE = 1
    MOVE = 2

  destinationParent = _messages.StringField(1)
  displayName = _messages.StringField(2)
  operationType = _messages.EnumField('OperationTypeValueValuesEnum', 3)
  sourceParent = _messages.StringField(4)


class FolderOperationError(_messages.Message):
  r"""A classification of the Folder Operation error.

  Enums:
    ErrorMessageIdValueValuesEnum: The type of operation error experienced.

  Fields:
    errorMessageId: The type of operation error experienced.
  """

  class ErrorMessageIdValueValuesEnum(_messages.Enum):
    r"""The type of operation error experienced.

    Values:
      ERROR_TYPE_UNSPECIFIED: The error type was unrecognized or unspecified.
      ACTIVE_FOLDER_HEIGHT_VIOLATION: The attempted action would violate the
        max folder depth constraint.
      MAX_CHILD_FOLDERS_VIOLATION: The attempted action would violate the max
        child folders constraint.
      FOLDER_NAME_UNIQUENESS_VIOLATION: The attempted action would violate the
        locally-unique folder display_name constraint.
      RESOURCE_DELETED_VIOLATION: The resource being moved has been deleted.
      PARENT_DELETED_VIOLATION: The resource a folder was being added to has
        been deleted.
      CYCLE_INTRODUCED_VIOLATION: The attempted action would introduce cycle
        in resource path.
      FOLDER_BEING_MOVED_VIOLATION: The attempted action would move a folder
        that is already being moved.
      FOLDER_TO_DELETE_NON_EMPTY_VIOLATION: The folder the caller is trying to
        delete contains active resources.
      DELETED_FOLDER_HEIGHT_VIOLATION: The attempted action would violate the
        max deleted folder depth constraint.
    """
    ERROR_TYPE_UNSPECIFIED = 0
    ACTIVE_FOLDER_HEIGHT_VIOLATION = 1
    MAX_CHILD_FOLDERS_VIOLATION = 2
    FOLDER_NAME_UNIQUENESS_VIOLATION = 3
    RESOURCE_DELETED_VIOLATION = 4
    PARENT_DELETED_VIOLATION = 5
    CYCLE_INTRODUCED_VIOLATION = 6
    FOLDER_BEING_MOVED_VIOLATION = 7
    FOLDER_TO_DELETE_NON_EMPTY_VIOLATION = 8
    DELETED_FOLDER_HEIGHT_VIOLATION = 9

  errorMessageId = _messages.EnumField('ErrorMessageIdValueValuesEnum', 1)


class GetIamPolicyRequest(_messages.Message):
  r"""Request message for `GetIamPolicy` method.

  Fields:
    options: OPTIONAL: A `GetPolicyOptions` object for specifying options to
      `GetIamPolicy`. This field is only used by Cloud IAM.
  """

  options = _messages.MessageField('GetPolicyOptions', 1)


class GetPolicyOptions(_messages.Message):
  r"""Encapsulates settings provided to GetIamPolicy.

  Fields:
    requestedPolicyVersion: Optional. The policy format version to be
      returned.  Valid values are 0, 1, and 3. Requests specifying an invalid
      value will be rejected.  Requests for policies with any conditional
      bindings must specify version 3. Policies without any conditional
      bindings may specify any valid value or leave the field unset.
  """

  requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)


class ListFoldersResponse(_messages.Message):
  r"""The ListFolders response message.

  Fields:
    folders: A possibly paginated list of Folders that are direct descendants
      of the specified parent resource.
    nextPageToken: A pagination token returned from a previous call to
      `ListFolders` that indicates from where listing should continue. This
      field is optional.
  """

  folders = _messages.MessageField('Folder', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class MoveFolderRequest(_messages.Message):
  r"""The MoveFolder request message.

  Fields:
    destinationParent: The resource name of the Folder or Organization to
      reparent the folder under. Must be of the form `folders/{folder_id}` or
      `organizations/{org_id}`.
  """

  destinationParent = _messages.StringField(1)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation.
      It typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata.  Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success.
      If the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`.  If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource.  For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name.  For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation.  It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata.  Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should have the format of `operations/some/unique/name`.
    response: The normal response of the operation in case of success.  If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`.  If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource.  For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name.  For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation.  It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata.  Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success.  If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`.  If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource.  For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name.  For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class Policy(_messages.Message):
  r"""Defines an Identity and Access Management (IAM) policy. It is used to
  specify access control policies for Cloud Platform resources.   A `Policy`
  consists of a list of `bindings`. A `binding` binds a list of `members` to a
  `role`, where the members can be user accounts, Google groups, Google
  domains, and service accounts. A `role` is a named list of permissions
  defined by IAM.  **JSON Example**      {       "bindings": [         {
  "role": "roles/owner",           "members": [
  "user:mike@example.com",             "group:admins@example.com",
  "domain:google.com",             "serviceAccount:my-other-
  app@appspot.gserviceaccount.com"           ]         },         {
  "role": "roles/viewer",           "members": ["user:sean@example.com"]
  }       ]     }  **YAML Example**      bindings:     - members:       -
  user:mike@example.com       - group:admins@example.com       -
  domain:google.com       - serviceAccount:my-other-
  app@appspot.gserviceaccount.com       role: roles/owner     - members:
  - user:sean@example.com       role: roles/viewer   For a description of IAM
  and its features, see the [IAM developer's
  guide](https://cloud.google.com/iam/docs).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members` to a `role`. `bindings` with no
      members will result in an error.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy.  If no `etag` is provided in the call to `setIamPolicy`, then
      the existing policy is overwritten.
    version: Specifies the format of the policy.  Valid values are 0, 1, and
      3. Requests specifying an invalid value will be rejected.  Policies with
      any conditional bindings must specify version 3. Policies without any
      conditional bindings may specify any valid value or leave the field
      unset.
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class ProjectCreationStatus(_messages.Message):
  r"""A status object which is used as the `metadata` field for the Operation
  returned by CreateProject. It provides insight for when significant phases
  of Project creation have completed.

  Fields:
    createTime: Creation time of the project creation workflow.
    gettable: True if the project can be retrieved using GetProject. No other
      operations on the project are guaranteed to work until the project
      creation is complete.
    ready: True if the project creation process is complete.
  """

  createTime = _messages.StringField(1)
  gettable = _messages.BooleanField(2)
  ready = _messages.BooleanField(3)


class SearchFoldersRequest(_messages.Message):
  r"""The request message for searching folders.

  Fields:
    pageSize: The maximum number of folders to return in the response. This
      field is optional.
    pageToken: A pagination token returned from a previous call to
      `SearchFolders` that indicates from where search should continue. This
      field is optional.
    query: Search criteria used to select the Folders to return. If no search
      criteria is specified then all accessible folders will be returned.
      Query expressions can be used to restrict results based upon
      displayName, lifecycleState and parent, where the operators `=`, `NOT`,
      `AND` and `OR` can be used along with the suffix wildcard symbol `*`.
      The displayName field in a query expression should use escaped quotes
      for values that include whitespace to prevent unexpected behavior.  Some
      example queries are:  * Query `displayName=Test*` returns Folder
      resources whose display name starts with "Test". * Query
      `lifecycleState=ACTIVE` returns Folder resources with `lifecycleState`
      set to `ACTIVE`. * Query `parent=folders/123` returns Folder resources
      that have `folders/123` as a parent resource. * Query
      `parent=folders/123 AND lifecycleState=ACTIVE` returns active Folder
      resources that have `folders/123` as a parent resource. * Query
      `displayName=\\"Test String\\"` returns Folder resources with display
      names that include both "Test" and "String".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  query = _messages.StringField(3)


class SearchFoldersResponse(_messages.Message):
  r"""The response message for searching folders.

  Fields:
    folders: A possibly paginated folder search results. the specified parent
      resource.
    nextPageToken: A pagination token returned from a previous call to
      `SearchFolders` that indicates from where searching should continue.
      This field is optional.
  """

  folders = _messages.MessageField('Folder', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Cloud Platform services (such as Projects)
      might reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: paths: "bindings, etag"
      This field is only used by Cloud IAM.
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details.  You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details.  There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as '*' or 'storage.*') are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


class UndeleteFolderRequest(_messages.Message):
  r"""The UndeleteFolder request message."""


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating instant snapshots."""


from googlecloudsdk.calliope import base

DETAILED_HELP = {
    'brief': 'Create, list and delete Compute Engine instant snapshots',
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.GA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.PREVIEW,
)
class InstantSnapshots(base.Group):
  """List, create, and delete Compute Engine instant snapshots."""


InstantSnapshots.category = base.INSTANCES_CATEGORY

InstantSnapshots.detailed_help = {
    'brief': 'Create, list and delete Compute Engine instant snapshots',
    'DESCRIPTION': """\
        List, create, describe, and delete Compute Engine instant snapshots.

        See also: [Snapshots API](https://cloud.google.com/compute/docs/reference/rest/beta/instantSnapshots).
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe worker pool command."""


from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.api_lib.cloudbuild.v2 import client_util as cloudbuild_v2_util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class Describe(base.DescribeCommand):
  """Describe a worker pool used by Cloud Build."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To get information about a worker pool named `wp1` in region `us-central1`, run:

            $ {command} wp1 --region=us-central1
          """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--region',
        help='The Cloud region where the worker pool is.')
    parser.add_argument(
        'WORKER_POOL', help='The ID of the worker pool to describe.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """

    return _DescribeWorkerPoolFirstGen(args, self.ReleaseTrack())


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribeBeta(Describe):
  """Describe a worker pool used by Cloud Build."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--region',
        help='The Cloud region where the worker pool is.')
    parser.add_argument(
        '--generation',
        default=1,
        type=int,
        help=('Generation of the worker pool.'))
    parser.add_argument(
        'WORKER_POOL', help='The ID of the worker pool to describe.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """

    if args.generation == 1:
      return _DescribeWorkerPoolFirstGen(args, self.ReleaseTrack())
    if args.generation == 2:
      return _DescribeWorkerPoolSecondGen(args)

    raise exceptions.InvalidArgumentException(
        '--generation',
        'please use one of the following valid generation values: 1, 2',
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(Describe):
  """Describe a worker pool used by Cloud Build."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument(
        '--region',
        help='The Cloud region where the worker pool is.')
    parser.add_argument(
        '--generation',
        default=1,
        type=int,
        help=('Generation of the worker pool.'))
    parser.add_argument(
        'WORKER_POOL', help='The ID of the worker pool to describe.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """

    if args.generation == 1:
      return _DescribeWorkerPoolFirstGen(args, self.ReleaseTrack())
    if args.generation == 2:
      return _DescribeWorkerPoolSecondGen(args)

    raise exceptions.InvalidArgumentException(
        '--generation',
        'please use one of the following valid generation values: 1, 2',
    )


def _DescribeWorkerPoolSecondGen(args):
  """Describes a Worker Pool Second Generation.

  Args:
    args: an argparse namespace. All the arguments that were provided to the
        create command invocation.

  Returns:
    A Worker Pool Second Generation resource.
  """
  client = cloudbuild_v2_util.GetClientInstance()
  messages = client.MESSAGES_MODULE

  wp_region = args.region
  if not wp_region:
    wp_region = properties.VALUES.builds.region.GetOrFail()

  # Get the workerpool second gen ref
  wp_resource = resources.REGISTRY.Parse(
      None,
      collection='cloudbuild.projects.locations.workerPoolSecondGen',
      api_version=cloudbuild_v2_util.GA_API_VERSION,
      params={
          'projectsId': properties.VALUES.core.project.Get(required=True),
          'locationsId': wp_region,
          'workerPoolSecondGenId': args.WORKER_POOL,
      })

  # Send the Get request
  wp = client.projects_locations_workerPoolSecondGen.Get(
      messages.CloudbuildProjectsLocationsWorkerPoolSecondGenGetRequest(
          name=wp_resource.RelativeName()))

  # Format the workerpool second gen name for display
  try:
    wp.name = cloudbuild_v2_util.WorkerPoolSecondGenShortName(wp.name)
  except ValueError:
    pass  # Must be an old version.

  return wp


def _DescribeWorkerPoolFirstGen(args, release_track):
  """Describes a Worker Pool First Generation.

  Args:
    args: an argparse namespace. All the arguments that were provided to the
        create command invocation.
    release_track: The desired value of the enum
      googlecloudsdk.calliope.base.ReleaseTrack.

  Returns:
    A Worker Pool First Generation resource.
  """
  wp_region = args.region
  if not wp_region:
    wp_region = properties.VALUES.builds.region.GetOrFail()

  client = cloudbuild_util.GetClientInstance(release_track)
  messages = cloudbuild_util.GetMessagesModule(release_track)

  parent = properties.VALUES.core.project.Get(required=True)

  wp_name = args.WORKER_POOL

  # Get the workerpool ref
  wp_resource = resources.REGISTRY.Parse(
      None,
      collection='cloudbuild.projects.locations.workerPools',
      api_version=cloudbuild_util.RELEASE_TRACK_TO_API_VERSION[release_track],
      params={
          'projectsId': parent,
          'locationsId': wp_region,
          'workerPoolsId': wp_name,
      })

  # Send the Get request
  wp = client.projects_locations_workerPools.Get(
      messages.CloudbuildProjectsLocationsWorkerPoolsGetRequest(
          name=wp_resource.RelativeName()))

  # Format the workerpool name for display
  try:
    wp.name = cloudbuild_util.WorkerPoolShortName(wp.name)
  except ValueError:
    pass  # Must be an old version.

  return wp



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for creating new member interconnects in an interconnect group."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.interconnects.groups import client
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute.interconnects import flags as interconnect_flags
from googlecloudsdk.command_lib.compute.interconnects.groups import flags
from googlecloudsdk.core import properties

DETAILED_HELP = {
    'DESCRIPTION': (
        """\
        *{command}* is used to create new member interconnects in an
        interconnect group.

        For an example, refer to the *EXAMPLES* section below.
        """
    ),
    'DEFAULTABLE PER-INTERCONNECT FLAGS': (
        """\
        All flags except `--intent-mismatch-behavior` in the *OPTIONAL FLAGS*
        section are defaultable per-interconnect. This means that you can
        set a default value by using these flags, or you can override the value
        for each interconnect by using the `--interconnect` flag.

        Furthermore, if the default values for the following flags are not
        specified, then they must be specified per-interconnect in the
        `--interconnect` flag:

        * `--facility`
        * `--interconnect-type`
        * `--link-type`
        * `--requested-link-count`
        """
    ),
    'EXAMPLES': (
        """\
        To create interconnects interconnect1 and interconnect2 in interconnect
        group example-interconnect-group, run:

          $ {command} example-interconnect-group --interconnect-type=DEDICATED
          --link-type=LINK_TYPE_ETHERNET_10G_LR --requested-link-count=1
          --facility=iad-1 --interconnect="name=interconnect1"
          --interconnect="name=interconnect2,facility=iad-5467"

        Note that facility is required for each member interconnect.
        interconnect1's facility is set to the default value iad-1 and
        interconnect2's facility is overridden to iad-5467 in the
        `--interconnect` flag.
        """
    ),
}

# Interconnect Flags
_FACILITY = 'facility'
_DESCRIPTION = 'description'
_NAME = 'name'
_LINK_TYPE = 'link-type'
_REQUESTED_LINK_COUNT = 'requested-link-count'
_INTERCONNECT_TYPE = 'interconnect-type'
_ADMIN_ENABLED = 'admin-enabled'
_NO_ADMIN_ENABLED = 'no-admin-enabled'
_NOC_CONTACT_EMAIL = 'noc-contact-email'
_CUSTOMER_NAME = 'customer-name'
_REMOTE_LOCATION = 'remote-location'
_REQUESTED_FEATURES = 'requested-features'


def _CheckRequiredFlagPresent(args, ic_args, flag_name):
  """Checks that flag is present in template args or interconnect args."""
  flag_present_in_template = getattr(args, flag_name.replace('-', '_'))
  if not flag_present_in_template and flag_name not in ic_args:
    raise exceptions.RequiredArgumentException(
        '--' + flag_name,
        flag_name
        + ' must be provided either in template via --'
        + flag_name
        + ' or for each member interconnect via --interconnect flag.',
    )


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class CreateMembers(base.UpdateCommand):
  """Create new member interconnects in a Compute Engine interconnect group.

  *{command}* creates new member interconnects in a Compute Engine interconnect
  group.
  """

  INTERCONNECT_GROUP_ARG = None
  REMOTE_LOCATION_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.INTERCONNECT_GROUP_ARG = flags.InterconnectGroupArgument(plural=False)
    cls.INTERCONNECT_GROUP_ARG.AddArgument(
        parser, operation_type='create members'
    )
    flags.AddMemberInterconnectsForCreateMembers(parser)
    flags.AddFacility(parser)
    flags.AddRemoteLocation(parser)
    flags.AddIntentMismatchBehavior(parser)
    interconnect_flags.AddCreateArgsForInterconnectGroupsCreateMembers(
        parser, required=False
    )

  def Collection(self):
    return 'compute.interconnectGroups'

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.INTERCONNECT_GROUP_ARG.ResolveAsResource(args, holder.resources)
    project = properties.VALUES.core.project.GetOrFail()

    interconnect_group = client.InterconnectGroup(
        ref, project, compute_client=holder.client, resources=holder.resources
    )

    messages = holder.client.messages
    template_interconnect = (
        interconnect_group.MakeInterconnectGroupsCreateMembersInterconnectInput(
            facility=args.facility,
            description=args.description,
            name=None,
            link_type=flags.GetLinkType(messages, args.link_type),
            requested_link_count=args.requested_link_count,
            interconnect_type=flags.GetInterconnectType(
                messages, args.interconnect_type
            ),
            admin_enabled=args.admin_enabled,
            noc_contact_email=args.noc_contact_email,
            customer_name=args.customer_name,
            remote_location=args.remote_location,
            requested_features=flags.GetRequestedFeatures(
                messages, args.requested_features
            ),
        )
    )
    member_interconnects = []
    for ic_args in args.interconnect:
      _CheckRequiredFlagPresent(args, ic_args, _FACILITY)
      _CheckRequiredFlagPresent(args, ic_args, _INTERCONNECT_TYPE)
      _CheckRequiredFlagPresent(args, ic_args, _LINK_TYPE)
      _CheckRequiredFlagPresent(args, ic_args, _REQUESTED_LINK_COUNT)
      if _ADMIN_ENABLED in ic_args:
        admin_enabled = True
      elif _NO_ADMIN_ENABLED in ic_args:
        admin_enabled = False
      else:
        admin_enabled = None
      member_interconnects.append(
          interconnect_group.MakeInterconnectGroupsCreateMembersInterconnectInput(
              facility=ic_args.get(_FACILITY),
              description=ic_args.get(_DESCRIPTION),
              name=ic_args.get(_NAME),
              link_type=flags.GetLinkType(messages, ic_args.get(_LINK_TYPE)),
              requested_link_count=ic_args.get(_REQUESTED_LINK_COUNT),
              interconnect_type=flags.GetInterconnectType(
                  messages, ic_args.get(_INTERCONNECT_TYPE)
              ),
              admin_enabled=admin_enabled,
              noc_contact_email=ic_args.get(_NOC_CONTACT_EMAIL),
              customer_name=ic_args.get(_CUSTOMER_NAME),
              remote_location=ic_args.get(_REMOTE_LOCATION),
              requested_features=flags.GetRequestedFeatures(
                  messages, ic_args.get(_REQUESTED_FEATURES)
              ),
          )
      )

    return interconnect_group.CreateMembers(
        intent_mismatch_behavior=flags.GetIntentMismatchBehavior(
            messages, args.intent_mismatch_behavior
        ),
        template_interconnect=template_interconnect,
        member_interconnects=member_interconnects,
    )


CreateMembers.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deleting network edge security services."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.api_lib.compute.network_edge_security_services import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.network_edge_security_services import flags


class Delete(base.DeleteCommand):
  r"""Delete network edge security services.

  *{command}* deletes Compute Engine network edge security services.

  ## EXAMPLES

  To delete a network edge security service with the name 'my-service' in region
  'us-central1', run:

    $ {command} my-service \
      --region=us-central1
  """

  NETWORK_EDGE_SECURITY_SERVICE_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.NETWORK_EDGE_SECURITY_SERVICE_ARG = (
        flags.NetworkEdgeSecurityServiceArgument())
    cls.NETWORK_EDGE_SECURITY_SERVICE_ARG.AddArgument(
        parser, operation_type='delete')
    parser.display_info.AddCacheUpdater(
        flags.NetworkEdgeSecurityServicesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    # Only one network edge security service can be present per region
    ref = self.NETWORK_EDGE_SECURITY_SERVICE_ARG.ResolveAsResource(
        args, holder.resources)
    utils.PromptForDeletion([ref])

    network_edge_security_service = client.NetworkEdgeSecurityService(
        ref, compute_client=holder.client)

    return network_edge_security_service.Delete()



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Listing Compute Engine commitments."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List Compute Engine commitments."""

  detailed_help = {
      'EXAMPLES': '''
        To list commitments, run:

          $ {command}
      '''
  }

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat("""\
        table(name,
              region.basename(),
              endTimestamp,
              status)
        """)
    parser.display_info.AddUriFunc(utils.MakeGetUriFunc())
    lister.AddRegionsArg(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    request_data = lister.ParseRegionalFlags(args, holder.resources)

    list_implementation = lister.RegionalLister(
        client, client.apitools_client.regionCommitments)

    return lister.Invoke(request_data, list_implementation)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for the SCC RemediationIntent resource commands."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers


# OrganizationId attribute flag for the resouce
def OrganizationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='organization',
      help_text='The Google Cloud organization for the {resource}.',
  )


# LocationId attribute flag for the resouce
def LocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='The Google Cloud location for the {resource}.',
  )


# RemediationIntentId attribute flag for the resouce
def RemediationIntentAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='remediation-intent',
      help_text='The remediation intent for the {resource}.',
  )


# Positional Resource flag
def GetRemediationIntentResourceSpec():
  return concepts.ResourceSpec(
      'securityposture.organizations.locations.remediationIntents',
      resource_name='remediationIntent',
      organizationsId=OrganizationAttributeConfig(),
      locationsId=LocationAttributeConfig(),
      remediationIntentsId=RemediationIntentAttributeConfig(),
      disable_auto_completers=False,
  )


# Add the remediation intent resource argument to the parser
def AddRemediationIntentResourceArg(parser):
  concept_parsers.ConceptParser.ForResource(
      'remediationintent',
      GetRemediationIntentResourceSpec(),
      'The remediation intent resource.',
      required=True,
  ).AddToParser(parser)

# Other flags (positional)
POSITIONAL_PARENT_NAME_FLAG = base.Argument(
    'parent',
    help=""" The parent resource to create the remediation intent under.
        Format: organizations/{org_id}/locations/{loc_id} """,
)


# Other flags (non-positional)
PARENT_NAME_FLAG = base.Argument(
    '--parent',
    help=""" The parent resource to create the remediation intent under.
        Format: organizations/{org_id}/locations/{loc_id} """,
    required=True,
)

FINDING_NAME_FLAG = base.Argument(
    '--finding-name',
    help=""" Canonical name of the SCC finding
        Format: projects/{proj_id}/sources/{src_id}/locations/{loc_id}/findings/{finding_id} """,
    required=False,
)

WORKFLOW_TYPE_FLAG = base.Argument(
    '--workflow-type',
    choices=['semi-autonomous', 'manual'],
    help=""" Type of the workflow to be created""",
    required=True,
)

ETAG_FLAG = base.Argument(
    '--etag',
    help=""" Etag is an optional flag. If the provided Etag doesn't match the server generated Etag, the operation won't proceed.""",
    required=False,
)

REMEDIATION_INTENT_FROM_FILE_FLAG = base.Argument(
    '--ri-from-file',
    help=""" Path to the YAML file containing the remediation intent resource.
              Format: /path/to/file.yaml.""",
    type=arg_parsers.YAMLFileContents(),
    required=True,
)

UPDATE_MASK_FLAG = base.Argument(
    '--update-mask',
    help=""" Comma separated string containing list of fields to be updated.
              Format: field1,field2.""",
    required=False,
)

GIT_CONFIG_FILE_PATH_FLAG = base.Argument(
    '--git-config-path',
    help=""" Path to the git config file in YAML format to raise the PR.
            Format: /path/to/file.yaml. Sample Config file:\n
              remote: origin
              main-branch-name: master
              branch-prefix: iac-remediation-
              reviewers: reviewer1,reviewer2
            """,
    metavar='GIT_SETTINGS',
    type=arg_parsers.YAMLFileContents(),
    required=True,
)

ORG_ID_FLAG = base.Argument(
    '--org-id',
    help=""" The Google Cloud organization ID""",
    required=True,
)

ROOT_DIR_PATH_FLAG = base.Argument(
    '--root-dir-path',
    help=""" The relative path to the root directory for the terraform
    repository.\n If not specified, the default value
    is assumed to be the current directory.""",
    required=False,
)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing backend buckets."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.backend_buckets import flags


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.DefaultUniverseOnly
class Describe(base.DescribeCommand):
  """Describe a backend bucket.

  *{command}* displays all data associated with a backend bucket in a
  project.
  """

  BACKEND_BUCKET_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.BACKEND_BUCKET_ARG = flags.GLOBAL_REGIONAL_BACKEND_BUCKET_ARG
    cls.BACKEND_BUCKET_ARG.AddArgument(parser, operation_type='describe')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    backend_bucket_ref = self.BACKEND_BUCKET_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client),
        default_scope=compute_scope.ScopeEnum.GLOBAL,
    )

    requests = []
    if backend_bucket_ref.Collection() == 'compute.backendBuckets':
      requests = [(
          client.apitools_client.backendBuckets,
          'Get',
          client.messages.ComputeBackendBucketsGetRequest(
              **backend_bucket_ref.AsDict()
          ),
      )]
    elif backend_bucket_ref.Collection() == 'compute.regionBackendBuckets':
      requests = [(
          client.apitools_client.regionBackendBuckets,
          'Get',
          client.messages.ComputeRegionBackendBucketsGetRequest(
              **backend_bucket_ref.AsDict()
          ),
      )]

    return client.MakeRequests(requests)[0]


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class DescribeBeta(Describe):
  """Describe a backend bucket.

  *{command}* displays all data associated with a backend bucket in a
  project.

  To describe a global backend bucket, run either of the following:

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1
        --global

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1

  To describe a regional backend bucket, run the following:

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1
        --region=us-central1
  """

  pass


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class DescribeAlpha(DescribeBeta):
  """Describe a backend bucket.

  *{command}* displays all data associated with a backend bucket in a
  project.

  To describe a global backend bucket, run either of the following:

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1
        --global

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1

  To describe a regional backend bucket, run the following:

      $ {command} my-backend-bucket
        --gcs-bucket-name gcs-bucket-1
        --region=us-central1
  """

  pass



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for bigtable instances get-iam-policy."""


import textwrap

from googlecloudsdk.api_lib.bigtable import instances
from googlecloudsdk.api_lib.bigtable import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bigtable import arguments


class GetIamPolicy(base.ListCommand):
  """Get the IAM policy for a Cloud Bigtable instance."""

  detailed_help = {
      'EXAMPLES':
          textwrap.dedent("""\
          To print the IAM policy for an instance, run:

            $ {command} my-instance-id

          """),
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    arguments.AddInstanceResourceArg(
        parser, 'to get the IAM policy for', positional=True)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      A IAM policy message.
    """
    instance_ref = util.GetInstanceRef(args.instance)
    return instances.GetIamPolicy(instance_ref)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Lists compatible accelerator profiles for GKE Inference Quickstart."""

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.ai.recommender import util
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import commands
from googlecloudsdk.command_lib.run.printers import profiles_csv_printer
from googlecloudsdk.command_lib.run.printers import profiles_printer_ga as profiles_printer
from googlecloudsdk.core.resource import resource_printer


_EXAMPLES = """
To list compatible accelerator profiles for a model, run:

$ {command} --model=deepseek-ai/DeepSeek-R1-Distill-Qwen-7B
"""


def decimal_to_amount(decimal_value):
  """Converts a decimal representation to an Amount proto."""

  units = int(decimal_value)
  nanos = int((decimal_value - units) * 1e9)

  return (units, nanos)


def amount_to_decimal(cost):
  """Converts cost to a decimal representation."""
  units = cost.units
  if not units:
    units = 0
  decimal_value = +(units + cost.nanos / 1e9)
  return f"{decimal_value:.3f}"


def get_decimal_cost(costs):
  """Returns the cost per million normalized output tokens as a decimal.

  Args:
    costs: The costs to convert.
  """
  output_token_cost = "N/A"
  if costs and costs[0].costPerMillionOutputTokens:
    output_token_cost = amount_to_decimal(
        costs[0].costPerMillionOutputTokens
    )
  input_token_cost = "N/A"
  if costs and costs[0].costPerMillionInputTokens:
    input_token_cost = amount_to_decimal(costs[0].costPerMillionInputTokens)
  return (input_token_cost, output_token_cost)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(commands.List):
  """List compatible accelerator profiles.

  This command lists all supported accelerators with their performance details.
  By default, the supported accelerators are displayed in a table format with
  select information for each accelerator. To see all details, use
  --format=yaml or --format=csvprofile.

  To get supported model, model servers, and model server versions, run `gcloud
  container ai profiles models list`, `gcloud container ai
  profiles model-servers list`, and `gcloud container ai profiles
  model-server-versions list`.
  """

  @staticmethod
  def Args(parser):
    parser.add_argument(
        "--model",
        help="The model.",
    )
    parser.add_argument(
        "--model-server",
        help="The model server.",
    )
    parser.add_argument(
        "--model-server-version",
        help=(
            "The model server version. If omitted, the most recently"
            " benchmarked version is used. Other options include the model"
            " server version of a profile, or 'all' to return all versions."
        ),
    )
    parser.add_argument(
        "--target-ntpot-milliseconds",
        type=int,
        help=(
            "The target normalized time per output token (NTPOT) in"
            " milliseconds. NTPOT is measured as the request_latency /"
            " output_tokens. If this field is set, the command will only return"
            " accelerators that can meet the target ntpot milliseconds and"
            " display their throughput performance at the target latency."
            " Otherwise, the command will return all accelerators and display"
            " their highest throughput performance."
        ),
    )
    parser.add_argument(
        "--target-ttft-milliseconds",
        type=int,
        help=(
            "The target time to first token (TTFT) in"
            " milliseconds. TTFT is measured as the request_latency /"
            " output_tokens. If this field is set, the command will only return"
            " profiles that can meet the target ttft milliseconds and"
            " display their throughput performance at the target latency."
            " Otherwise, the command will return all profiles and display"
            " their highest throughput performance."
        ),
    )
    parser.add_argument(
        "--target-itl-milliseconds",
        type=int,
        help=(
            "If specified, results will only show profiles with instance types"
            " that can meet the latency target and will show their throughput"
            " performances at the target inter-token latency (ITL)."
        ),
    )
    parser.add_argument(
        "--target-cost-per-million-output-tokens",
        type=float,
        required=False,
        help=(
            "The target cost per million output tokens to filter profiles by,"
            " unit is 1 USD up to 5 decimal places."
        ),
    )
    parser.add_argument(
        "--target-cost-per-million-input-tokens",
        type=float,
        required=False,
        help=(
            "The target cost per million input tokens to filter profiles by,"
            " unit is 1 USD up to 5 decimal places."
        ),
    )
    parser.add_argument(
        "--pricing-model",
        required=False,
        type=str,
        help=(
            "The pricing model to use to calculate token cost. Currently, this"
            " supports on-demand, spot, 3-years-cud, 1-year-cud"
        ),
    )
    parser.add_argument(
        "--format",
        help=(
            "The output format. Default is profile, which displays the profile"
            " information in a table format, including cost conversions."
            " csvprofile displays the profile information in a CSV"
            " format.Options include csvprofile, profile, and yaml. "
        ),
    )
    parser.add_argument(
        "--use-case",
        required=False,
        type=str,
        help=(
            " If specified, results will only show profiles that match the"
            " provided use case. Options are: Advanced Customer Support, Code"
            " Completion, Text Summarization, Chatbot (ShareGPT), Text"
            " Generation, Deep Research"
        ),
    )
    parser.add_argument(
        "--target-input-length",
        required=False,
        type=int,
        help=(
            " If specified, results will only show profiles that have an input"
            " length within 20% of the specified one. Only works alongside"
            " output length."
        ),
    )
    parser.add_argument(
        "--target-output-length",
        required=False,
        type=int,
        help=(
            "If specified, results will only show profiles that have an output"
            " length within 20% of the specified one. Only works alongside"
            " input length."
        ),
    )
    parser.add_argument(
        "--serving-stack",
        required=False,
        help=(
            "The serving stack to filter profiles by. If not"
            " provided, profiles for all serving stacks that support"
            " the given model and model server will be returned."
        ),
    )
    parser.add_argument(
        "--serving-stack-version",
        required=False,
        help=(
            "The serving stack version. If omitted, the most recently"
            " benchmarked version is used. Other options include the serving"
            " stack version of a profile, or 'all' to return all versions."
        ),
    )
    resource_printer.RegisterFormatter(
        profiles_printer.PROFILES_PRINTER_FORMAT,
        profiles_printer.ProfilePrinter,
    )
    resource_printer.RegisterFormatter(
        profiles_csv_printer.PROFILES_PRINTER_FORMAT,
        profiles_csv_printer.ProfileCSVPrinter,
    )
    parser.display_info.AddFormat(profiles_printer.PROFILES_PRINTER_FORMAT)

  def Run(self, args):
    client = util.GetClientInstance(base.ReleaseTrack.GA)
    messages = util.GetMessagesModule(base.ReleaseTrack.GA)

    performance_requirements = messages.PerformanceRequirements()
    workload_spec = messages.WorkloadSpec()
    if args.target_ntpot_milliseconds:
      performance_requirements.targetNtpotMilliseconds = (
          args.target_ntpot_milliseconds
      )
    if args.target_ttft_milliseconds:
      performance_requirements.targetTtftMilliseconds = (
          args.target_ttft_milliseconds
      )
    if args.target_itl_milliseconds:
      performance_requirements.targetItlMilliseconds = (
          args.target_itl_milliseconds
      )
    if args.use_case:
      workload_spec.useCase = (
          args.use_case
      )
    if args.target_input_length:
      workload_spec.averageInputLength = (
          args.target_input_length
      )
    if args.target_output_length:
      workload_spec.averageOutputLength = (
          args.target_output_length
      )
    if (
        args.target_cost_per_million_output_tokens
        or args.target_cost_per_million_input_tokens
        or args.pricing_model
    ):
      cost = messages.Cost()
      if args.target_cost_per_million_output_tokens:
        units, nanos = decimal_to_amount(
            args.target_cost_per_million_output_tokens
        )
        cost.costPerMillionOutputTokens = messages.Amount(
            units=units, nanos=nanos
        )
      if args.target_cost_per_million_input_tokens:
        units, nanos = decimal_to_amount(
            args.target_cost_per_million_input_tokens
        )
        cost.costPerMillionInputTokens = messages.Amount(
            units=units, nanos=nanos
        )
      if args.pricing_model:
        cost.pricingModel = args.pricing_model
      performance_requirements.targetCost = cost

    serving_stack = None
    if args.serving_stack:
      serving_stack = messages.ServingStack(
          name=args.serving_stack,
      )
      if args.serving_stack_version:
        serving_stack.version = args.serving_stack_version

    try:
      request = messages.FetchProfilesRequest(
          model=args.model,
          modelServer=args.model_server,
          modelServerVersion=args.model_server_version,
          servingStack=serving_stack,
      )
      if (
          performance_requirements.targetNtpotMilliseconds is not None
          or performance_requirements.targetTtftMilliseconds is not None
          or performance_requirements.targetCost is not None
      ):
        request.performanceRequirements = performance_requirements
      if (
          workload_spec.useCase is not None
          or workload_spec.averageInputLength is not None
          or workload_spec.averageOutputLength is not None
      ):
        request.workloadSpec = workload_spec
      response = client.profiles.Fetch(request)
      if response.profile:
        return response.profile
      else:
        return []
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)



"""Generated message classes for connectgateway version v1beta1.

The Connect Gateway service allows connectivity from external parties to
connected Kubernetes clusters.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'connectgateway'


class ConnectgatewayProjectsLocationsMembershipsGenerateCredentialsRequest(_messages.Message):
  r"""A ConnectgatewayProjectsLocationsMembershipsGenerateCredentialsRequest
  object.

  Enums:
    OperatingSystemValueValuesEnum: Optional. The operating system on which
      the generated kubeconfig will be used.

  Fields:
    applicationDefaultCredentials: Optional. Configures the generated
      kubeconfig to use Application Default Credentials. When this field is
      set to `true`, the `kubeconfig` generated by Connect Gateway will
      include `--use_application_default_credentials` in the arguments for
      `gke-gcloud-auth-plugin`. If `false`, the plugin will use the default
      gcloud credentials.
    forceUseAgent: Optional. Whether to force the use of Connect Agent-based
      transport in the generated kubeconfig. This will return a configuration
      that uses Connect Agent as the underlying transport mechanism for
      cluster types that would otherwise have used a different transport.
      Requires that Connect Agent be installed on the cluster. Setting this
      field to false is equivalent to not setting it.
    impersonatedServiceAccount: Optional. Service account to impersonate when
      using the generated kubeconfig. This should only be specified if all
      calls using this kubeconfig should be made using impersonation of the
      same service account.
    kubernetesNamespace: Optional. The namespace to use in the generated
      kubeconfig context. If this field is specified, the server will set the
      `namespace` field in kubeconfig context. If not specified, the
      `namespace` field is omitted.
    name: Required. The Fleet membership resource.
    operatingSystem: Optional. The operating system on which the generated
      kubeconfig will be used.
    version: Optional. The Connect Gateway version to be used in the generated
      kubeconfig. Leave this field blank to let the server choose the version
      (recommended).
  """

  class OperatingSystemValueValuesEnum(_messages.Enum):
    r"""Optional. The operating system on which the generated kubeconfig will
    be used.

    Values:
      OPERATING_SYSTEM_UNSPECIFIED: Generates a kubeconfig that works for all
        operating systems not defined below.
      OPERATING_SYSTEM_WINDOWS: Generates a kubeconfig that is specifically
        designed to work with Windows.
    """
    OPERATING_SYSTEM_UNSPECIFIED = 0
    OPERATING_SYSTEM_WINDOWS = 1

  applicationDefaultCredentials = _messages.BooleanField(1)
  forceUseAgent = _messages.BooleanField(2)
  impersonatedServiceAccount = _messages.StringField(3)
  kubernetesNamespace = _messages.StringField(4)
  name = _messages.StringField(5, required=True)
  operatingSystem = _messages.EnumField('OperatingSystemValueValuesEnum', 6)
  version = _messages.StringField(7)


class GenerateCredentialsResponse(_messages.Message):
  r"""Connection information for a particular membership.

  Fields:
    endpoint: The generated URI of the cluster as accessed through the Connect
      Gateway API.
    kubeconfig: A full YAML kubeconfig in serialized format.
  """

  endpoint = _messages.StringField(1)
  kubeconfig = _messages.BytesField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""GKE Hub Policy Controller library to manipulate API proto messages."""


from typing import Any, Dict


def additional_properties_to_dict(spec):
  """Extracts the additional properties of a message.Message as a dictionary."""
  if spec is None:
    return {}
  return {prop.key: prop.value for prop in spec.additionalProperties}


def set_additional_properties(message, additional_properties: Dict[Any, Any]):
  """Sets additional properties on a message."""
  ls = [
      message.AdditionalProperty(key=key, value=value)
      for key, value in additional_properties.items()
  ]
  message.additionalProperties = ls
  return message
